
-- # import Property
-- # import E
-- # import GHC.Base
-- # import GHC.Base-Models
-- # import Data.Maybe
-- # import Data.Maybe-Models
-- # import CoverPrelude

{------------------------------------------------------------------------
    Models
 ------------------------------------------------------------------------}

-- Expr -------------------------------------------------------------------

data Expr : Set where
  Val	: Int -> Expr
  Add	: Expr -> Expr -> Expr
  Throw : Expr
  Catch : Expr -> Expr -> Expr

injExpr : Expr -> "E:Expr"
injExpr (Val n)	  = "E.Val" n
injExpr (Add x y)	  = "E.Add" (injExpr x) (injExpr y)
injExpr Throw	  = "E.Throw"
injExpr (Catch x y)  = "E.Catch" (injExpr x) (injExpr y)

-- Op ---------------------------------------------------------------------

data Op : Set where
  PUSH	  : Int -> Op
  ADD	  : Op
  THROW	  : Op
  MARK	  : List Op -> Op
  UNMARK  : Op

injOp : Op -> "E:Op"
injOp (PUSH n)	= "E.PUSH" (injInt n)
injOp ADD	= "E.ADD"
injOp THROW	= "E.THROW"
injOp (MARK xs)	= "E.MARK" (injList injOp xs)
injOp UNMARK	= "E.UNMARK"

Code	= List Op
injCode = injList injOp

data Item : Set where
  VAL : Int -> Item
  HAN : Code -> Item

injItem : Item -> "E:Item"
injItem (VAL n)	  = "E.VAL" (injInt n)
injItem (HAN xs)  = "E.HAN" (injCode xs)

Stack	  = List Item
injStack  = injList injItem

-- eval -------------------------------------------------------------------

eval : Expr -> Maybe Int
eval (Val n)	= Just n
eval (Add x y)	= maybe Nothing (\x -> eval x + eval y

"eval model" : (e : Expr) -> eval e == "E.eval" (injExpr e)
"eval model" (Val n)	= cover-plugin( include "E" )
"eval model" (Add x y)	= cover-plugin( include "E"
				      , include "GHC.Num"
				      , "eval model" x
				      , "eval model" y
				      )

{-
-- comp -------------------------------------------------------------------

comp : Expr -> Code
comp (Val n)	= [PUSH n]
comp (Add x y)	= comp x ++ comp y ++ [ADD]

"comp model" : (e : Expr) -> injCode (comp e) == "E.comp" (injExpr e)
"comp model" (Val n)	= cover-plugin( include "E" )
"comp model" (Add x y)	= cover-plugin( include "E"
				      , "comp model" x
				      , "comp model" y
				      , rem
				      )
  where
    rem : (a, b : Code) -> injCode (a ++ b ++ [ADD]) ==
			      "GHC.Base.++" (injCode a) 
				("GHC.Base.++" (injCode b)
				    ("GHC.Base.:" "E.ADD" "GHC.Base.[]")
				)
    rem a b = cover-plugin(injList, map, injOp, "++ model", "map ++")

-- exec -------------------------------------------------------------------

pstack : Partial Stack -> "GHC.Base:[]" Int
pstack x = partial (lift injStack x)

exec : Stack -> Code -> Partial Stack
exec s []		      = val s
exec s (PUSH n :: ops)	      = exec (n :: s) ops
exec (m :: n :: s) (ADD :: ops)  = exec (n + m :: s) ops
exec [n] (ADD :: ops)	      = bot
exec [] (ADD :: ops)	      = bot

"exec model" : (s : Stack) -> (ops : Code) ->
		pstack (exec s ops) == "E.exec" (injStack s) (injCode ops)
"exec model" s []		      = cover-plugin( include "E" )
"exec model" s (PUSH n :: ops)	      = cover-plugin( include "E"
						    , "exec model" (n :: s) ops
						    )
"exec model" (m :: n :: s) (ADD :: ops)  = cover-plugin( include "E"
						    , include "GHC.Num"
						    , "exec model" (n + m :: s) ops
						    )
"exec model" [n] (ADD :: ops)	      = cover-plugin(include "E")
"exec model" [] (ADD :: ops)	      = cover-plugin(include "E")


{------------------------------------------------------------------------
    Proof
 ------------------------------------------------------------------------}

thm : (e : Expr) -> (s : Stack) -> (ops : Code) ->
       exec s (comp e ++ ops) == exec (eval e :: s) ops
thm (Val n) s ops     = fol-plugin()
thm (Add e1 e2) s ops = fol-plugin( thm e1, thm e2
				  , "++ assoc"
				  , (++)
				  , exec
				  )

cor : (e : Expr) -> (s : Stack) -> (ops : Code) ->
       let e'	= injExpr e
	   s'	= injStack s
	   ops'	= injCode ops
       in
	  "E.exec" s' ("GHC.Base.++" ("E.comp" e') ops') ==
	  "E.exec" ("GHC.Base.:" ("E.eval" e') s') ops'
cor e s ops = fol-plugin( thm, "eval model", "comp model"
			, "exec model"
			, "++ model"
			, "map ++"
			, injList
			)
-}
