{-| Definitions used by the compiled code.
-}

module AgdaLightPrelude where

data AgdaValue c
    = NeverMind String
    | Lam String (AgdaValue c -> AgdaValue c)
    | App (AgdaValue c) (AgdaValue c)
    | Fst (AgdaValue c)
    | Snd (AgdaValue c)
    | Pair (AgdaValue c) (AgdaValue c)
    | NewConstant c

instance Show c => Show (AgdaValue c) where
    showsPrec p v =
	case v of
	    NeverMind x	    -> showString x
	    Lam _ _	    ->
		showParen (p>0) $
		    showString "\\ " . showString (unwords xs) . showString " -> " .
		    shows w
		where
		    (xs,w) = lamWalk v
	    App v w	    ->
		showParen (p>9) $
		    showsPrec 9 v . showString " " . showsPrec 10 w
	    Fst v	    -> showsPrec 10 v . showString ".1"
	    Snd v	    -> showsPrec 10 v . showString ".2"
	    Pair v w	    ->
		showParen True $ shows v . showString ", " . shows w
	    NewConstant x   -> shows x
	where
	    lamWalk (Lam x f) = (x:xs,v)
		where
		    (xs,v) = lamWalk (f $ NeverMind x)
	    lamWalk v = ([],v)

app :: AgdaValue c -> AgdaValue c -> AgdaValue c
app (Lam _ f) w	= f w
app v w		= App v w

first :: AgdaValue c -> AgdaValue c
first (Pair v w)    = v
first v		    = Fst v

second :: AgdaValue c -> AgdaValue c
second (Pair v w)   = w
second v	    = Snd v

