%% The only subsection in this section
%\subsection{Category Theory}

One application of the FOL plug-in is to category theory. Typically, proofs in
category contain a fair amount of symbolic manipulation, something which we can
leave to the plug-in.

To reason about category theory we introduce the appropriate constants together
with their axioms.

\begin{code}
-- # import ExamplePrelude

Obj : Set
Hom : Obj -> Obj -> Set

id (a : Obj) : Hom a a
(@) (a, b, c : Obj) : Hom b c -> Hom a b -> Hom a c

-- We don't use these axioms
axId1 (a, b : Obj) : (f : Hom a b) -> f == id @ f
axId2 (a, b : Obj) : (f : Hom a b) -> f == f @ id

assoc (a, b, c, d : Obj) :
      (f:Hom c d) -> (g:Hom b c) -> (h:Hom a b) ->
      (f @ g) @ h == f @ (g @ h)
\end{code}

Now we can define what it means for a morphism to be {\em epi} and prove that if
the composition of two morphisms is epi then the first morphism must also be epi.

\begin{code}
isEpi (a, b : Obj) : Hom a b -> Prop
isEpi {_} {b} f = (c : Obj) -> (g, h : Hom b c) ->
                  g @ f == h @ f  ==>  g == h

epiI (a, b : Obj)(f : Hom a b) : isEpi f -> isEpi f


prop (a, b, c : Obj) : (f : Hom b c) -> (k : Hom a b) ->
			 isEpi (f @ k) ==> isEpi f
prop f k = impI (\epi_kf -> fol-plugin(assoc, epi_kf))
\end{code}
% prop f k = impI (\epi_kf c g h -> 
%                   fol-plugin(assoc g f k, assoc h f k, epi_kf c g h))

Gandalf has no problem proving this (very simple) proposition and, more importantly,
the proof that Gandalf produces is very close the proof we would write by hand.
Pretty printed, the proof we get looks as follows.
\[\begin{array}{lrll}
  (1) & \forall X\,Y\,Z. & (X\circ Y)\circ Z = X\circ(Y\circ Z) \\
  (2) & \forall X\,Y. & X\circ(f\circ k) = Y\circ(f\circ k) \Longrightarrow X = Y \\
  (3) & & g\circ f == h\circ f \\
  (4) & & g == h \Longrightarrow \bot \\
  (5) & \forall X. & g\circ(f\circ X) == h\circ(f\circ X) & \{ (1), (3) \} \\
  (6) & & \bot & \{ (2), (4), (5) \} \\
\end{array}\]

%    7 (X @ Y) @ Z == X @ (Y @ Z)
%    8 X @ (f @ k) == Y @ (f @ k)  ==>  X == Y
%    9 g0 @ f == h1 @ f
%    10 g0 == h1  ==>  _|_
%    11 g0 @ (f @ X) == h1 @ (f @ X) [para::9.1.2,7.1.1.1.2,demod::7]
%    12 _|_ [hyper::8,11,cut::10]

% See the appendix for an example involving algebra and induction.
